@@ -11,14 +11,19 @@ fn pulsify_max_pool(
     target: &mut PulsedModel,
     mapping: &HashMap<OutletId, OutletId>,
     _pulse: usize,
-) -> TractResult<TVec<OutletId>> {
+) -> TractResult<Option<TVec<OutletId>>> {
     fn min_value<D: Datum + tract_core::num_traits::Bounded>() -> Tensor {
         tensor0(D::min_value())
     }
     let fact = target.outlet_fact(mapping[&node.inputs[0]])?;
     let min = dispatch_numbers!(min_value(fact.datum_type)());
-    let (wire, pool_spec) = pulsify(&op.pool_spec, source, node, target, mapping, Some(min))?;
-    target.wire_node(&node.name, MaxPool { pool_spec, ..op.clone() }, &[wire])
+    if let Some((wire, pool_spec)) =
+        pulsify_pooled_input(&op.pool_spec, source, node, target, mapping, Some(min))?
+    {
+        Ok(Some(target.wire_node(&node.name, MaxPool { pool_spec, ..op.clone() }, &[wire])?))
+    } else {
+        Ok(None)
+    }
 }
 
 fn pulsify_sum_pool(
@@ -28,9 +33,14 @@ fn pulsify_sum_pool(
     target: &mut PulsedModel,
     mapping: &HashMap<OutletId, OutletId>,
     _pulse: usize,
-) -> TractResult<TVec<OutletId>> {
-    let (wire, pool_spec) = pulsify(&op.pool_spec, source, node, target, mapping, None)?;
-    target.wire_node(&node.name, SumPool { pool_spec, ..op.clone() }, &[wire])
+) -> TractResult<Option<TVec<OutletId>>> {
+    if let Some((wire, pool_spec)) =
+        pulsify_pooled_input(&op.pool_spec, source, node, target, mapping, None)?
+    {
+        Ok(Some(target.wire_node(&node.name, SumPool { pool_spec, ..op.clone() }, &[wire])?))
+    } else {
+        Ok(None)
+    }
 }
 
 impl PulsedOp for SumPool {
@@ -85,19 +95,19 @@ pub fn pulsed_output_facts(
     Ok(tvec!(fact))
 }
 
-pub fn pulsify(
+pub fn pulsify_pooled_input(
     spec: &PoolSpec,
     _source: &TypedModel,
     node: &TypedNode,
     target: &mut PulsedModel,
     mapping: &HashMap<OutletId, OutletId>,
     padding_value: Option<Tensor>,
-) -> TractResult<(OutletId, PoolSpec)> {
+) -> TractResult<Option<(OutletId, PoolSpec)>> {
     let mut wire = mapping[&node.inputs[0]];
     let mut fact: PulsedFact = target.outlet_fact(wire)?.clone();
     let input_shape = spec.data_format.shape(fact.shape.clone())?;
     if Some(fact.axis) == input_shape.n_axis() {
-        return Ok((wire, spec.clone()));
+        return Ok(None);
     }
     if fact.axis == input_shape.c_axis() {
         bail!("Can not pulsify cnn pooling ops along the input channel axis");
@@ -132,7 +142,7 @@ pub fn pulsify(
         if extra_delay > 0 {
             wire = target.wire_node(
                 format!("{}.delay-for-pad", node.name),
-                tract_pulse_opl::ops::Delay::new(fact.axis, &(&fact).into(), extra_delay, 0),
+                tract_pulse_opl::ops::Delay::new_typed(&(&fact).into(), fact.axis, extra_delay, 0),
                 &[wire],
             )?[0];
             fact = target.outlet_fact(wire)?.clone();
@@ -156,11 +166,11 @@ pub fn pulsify(
     let misalignment = fact.delay % pulse;
 
     if overlap > 0 || misalignment > 0 {
-        let align_to = (overlap + fact.delay).div_ceil(stride) * stride;
+        let align_to = (overlap + fact.delay).divceil(stride) * stride;
         let delay = align_to - overlap - fact.delay;
         wire = target.wire_node(
             format!("{}.delay", node.name),
-            tract_pulse_opl::ops::Delay::new(fact.axis, &(&fact).into(), delay, overlap),
+            tract_pulse_opl::ops::Delay::new_typed(&(&fact).into(), fact.axis, delay, overlap),
             &[wire],
         )?[0];
     }
@@ -184,8 +194,8 @@ pub fn pulsify(
                 aft.push(c.pad_after.to_usize()?);
             };
         }
-        Ok((wire, PoolSpec { padding: PaddingSpec::Explicit(bef, aft, true), ..spec.clone() }))
+        Ok(Some((wire, PoolSpec { padding: PaddingSpec::Explicit(bef, aft, true), ..spec.clone() })))
     } else {
-        Ok((wire, spec.clone()))
+        Ok(Some((wire, spec.clone())))
     }
 }
